#! /usr/bin/env python
# (C) 2012: Hans Georg Schaathun <georg@schaathun.net> 

# Create a config file for the HOLMES feature set

L1 = [ (3,1,3,4),
      (3,2,3,3),
      (4,2,3,3), (4,3,3,3),
      (5,2,3,3), (5,6,3,3),
      (6,10,3,3), (7,20,3,3),
      (3,1,4,1), (3,2,4,1),
      (4,2,4,1), (4,3,4,1),
      (5,2,4,1), (5,6,4,1),
      (6,6,4,1), (6,10,4,1),
      (7,10,4,1), (7,20,4,1) ]
L2 = [ (3,2,5,1), (2,1,4,2), (2,2,4,2), ]

V = []
global D
D = 0

def printPset(c,p):
   print "; Parameter set %s" % (p,)
   printOne(c,"HV", "MARKOV features horizontal/vertical",p)
   printOne(c,"D", "MARKOV features diagonal",p)
   printOne(c,"MAX", "MINMAX (MAX) features",p)
   printOne(c,"MIN", "MINMAX (MIN) features",p)

def printMMset(c,p):
   print "; Parameter set %s" % (p,)
   printOne(c,"MAX", "MINMAX (MAX) features",p)
   printOne(c,"MIN", "MINMAX (MIN) features",p)

def printSQset(c,p):
   print "; Parameter set %s" % (p,)
   printOne(c,"SQmax", "SQUARE (MAX) features",p)
   printOne(c,"SQmin", "SQUARE (MIN) features",p)
def printCALIset(c,p):
   print "; Parameter set %s" % (p,)
   printOne(c,"CALImax", "CALI (MAX) features",p)
   printOne(c,"CALImin", "CALI (MIN) features",p)

def printEDGE(c,p):
   print "; Parameter set %s" % (p,)
   printOne(c,"EDGEmax", "EDGE (MAX) features",p)
   printOne(c,"EDGEmin", "EDGE (MIN) features",p)

def printOne(c,sh,st,p):
   (s,q,m,T) = p
   key = "HM%02i%s" % (c,sh,)
   printgen(key,m,T)
   print "func = lambda im : Holmes%s( im, s=%s, q=%s, m=%s, T=%s )" % (sh,s,q,m,T)
   print "description = Holmes %s with parameters %s" % (st,p,)

def printKB(q):
   (m,T) = (3,4)
   key = "HKB%02i" % (q,)
   printgen(key,m,T)
   print "func = lambda im : HolmesKB( im, q=%s )" % (q,)
   print "description = Holmes KB features with quantisation factor %s" % (q,)
def printSUM(c,st,m,T):
   key = "HSUM%02i%s" % (c,st)
   printgen(key,m,T)
   print "func = lambda im : Holmes%s( im, m=%s, T=%s )" % (st,m,T,)
   print "description = Holmes %s features with m=%s, T=%s" % (st,m,T,)
def printgen(key,m,T):
   global D
   print ";rem;", V, D
   V.append(key)
   print "[%s]" % (key)
   print "scheme = sym"
   print "limit = %s" % (T,)
   print "dimension = %s" % (m,)
   c = (2*T+1)**m
   D += c
   print "count = %s" % ( c, )

### Feature creation

count = 1

print "# HOLMES features"
print "# This file is autogenerated by the mkholmes.py script."
print

print "; HOLMES Group 1 (MINMAX and MARKOV)"
for p in L1:
   printPset(count,p)
   count += 1
print "; HOLMES Group 2-3 (just MINMAX)"
for p in L2:
   printMMset(count,p)
   count += 1

print "; HOLMES Group 4 (KB)"
for q in [ 1,2,4 ]:
   printKB(q)

print "; HOLMES Group 5 (SQUARE)"
printSQset(0,(3,2,4,1))

print "; HOLMES Group 6 (CALI)"
printCALIset(1,(3,2,3,4))
printCALIset(2,(4,2,3,4))

print "; HOLMES Group 7 (EDGE)"
for q in [ 1,2,4 ]:
   printEDGE(q,(6,q,3,4))

print "; HOLMES Group 8 (MINMAX/MARKOV summed)"
for (c,m,T) in [ (1,3,4), (2,4,2) ]:
   printSUM(c,"HVsum",m,T)
   printSUM(c,"Dsum",m,T)
   printSUM(c,"MINsum",m,T)
   printSUM(c,"MAXsum",m,T)

print "; feature vectors"
print "[HOLMES-33k]"
print "vector = True"
print "features =", " ".join(V)
print "description = The 33k-dimensional HOLMES feature vector of Fridrich et al."
print "dim = %i" % (D,)
